#include "ktimezones.h"
#include <kapplication.h>
#include <qdatetime.h>

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
    KInstance instance("ktimezonestest");
    unsigned error = 1;

    if ((argc==2) && (strcmp(argv[1], "local")==0))
    {
        error = 0;
        KSystemTimezones timezones;
        QTime t;
        int milliseconds1;
        int milliseconds2;

        // Find the local timezone.
        t.start();
        const KTimezone *timezone = timezones.local();
        milliseconds1 = t.elapsed();
        printf( "Local timezone: %s\n", timezone->name().latin1() );
        t.start();
        timezone = timezones.local();
        milliseconds2 = t.elapsed();
        if (milliseconds2 > milliseconds1)
        {
            printf( "First local() call took: %u milliseconds\n", milliseconds1 );
            printf( "Second, cached local() call took: %u milliseconds\n", milliseconds2 );
            error++;
        }

        // Find the current offset of the UTC timezone.
        timezone = timezones.zone("UTC");
        if (0 != timezone->offset(QDateTime::currentDateTime()))
        {
            printf( "UTC timezone offset should be 0: %d\n", timezone->offset(QDateTime::currentDateTime()) );
            error++;
        }

        // Find some offsets for Europe/London.
        char *london = "Europe/London";
        timezone = timezones.zone(london);
        QDateTime winter(QDateTime::fromString("2005-01-01T00:00:00", Qt::ISODate));
        QDateTime summer(QDateTime::fromString("2005-06-01T00:00:00", Qt::ISODate));
        if (0 != timezone->offset(winter))
        {
            printf( "%s winter timezone offset should be 0: %d\n", london, timezone->offset(winter) );
            error++;
        }
        if (3600 != timezone->offset(summer))
        {
            printf( "%s summer timezone offset should be 3600: %d\n", london, timezone->offset(summer) );
            error++;
        }

        // Try timezone conversions.
        const KTimezone *losAngeles = timezones.zone("America/Los_Angeles");
        char *bstBeforePdt = "2005-03-28T00:00:00";
        char *bstAfterPdt = "2005-05-01T00:00:00";
        char *gmtBeforePst = "2005-10-30T01:00:00";
        char *gmtAfterPst = "2005-12-01T00:00:00";
        QString result;
        result = timezone->convert(losAngeles, QDateTime::fromString(bstBeforePdt, Qt::ISODate)).toString(Qt::ISODate);
        if ("2005-03-27T15:00:00" != result)
        {
            printf( "BST before PDT, %s should be 2005-03-27T15:00:00: %s\n", bstBeforePdt, result.latin1() );
            error++;
        }
        result = timezone->convert(losAngeles, QDateTime::fromString(bstAfterPdt, Qt::ISODate)).toString(Qt::ISODate);
        if ("2005-04-30T16:00:00" != result)
        {
            printf( "BST and PDT,    %s should be 2005-04-30T16:00:00: %s\n", bstAfterPdt, result.latin1() );
            error++;
        }
        result = timezone->convert(losAngeles, QDateTime::fromString(gmtBeforePst, Qt::ISODate)).toString(Qt::ISODate);
        if ("2005-10-29T17:00:00" != result)
        {
            printf( "GMT before PST, %s should be 2005-10-29T17:00:00: %s\n", gmtBeforePst, result.latin1() );
            error++;
        }
        result = timezone->convert(losAngeles, QDateTime::fromString(gmtAfterPst, Qt::ISODate)).toString(Qt::ISODate);
        if ("2005-11-30T16:00:00" != result)
        {
            printf( "GMT and PST,    %s should be 2005-11-30T16:00:00: %s\n", gmtAfterPst, result.latin1() );
            error++;
        }
        if (!KTimezone::isValidLatitude(89.0))
        {
            printf( "Latitude 89 should be valid\n");
            error++;
        }
        if (KTimezone::isValidLatitude(91.0))
        {
            printf( "Latitude 91 should be invalid\n");
            error++;
        }
        if (!KTimezone::isValidLongitude(179.0))
        {
            printf( "Longitude 179 should be valid\n");
            error++;
        }
        if (KTimezone::isValidLongitude(181.0))
        {
            printf( "Longitude 181 should be valid\n");
            error++;
        }
        return error ? 1 : 0;
    }

    if ((argc==2) && (strcmp(argv[1], "all")==0))
    {
        error = 0;
        KSystemTimezones timezones;
        KTimezones::ZoneMap allZones = timezones.allZones();
        unsigned zones = 0;
        for ( KTimezones::ZoneMap::const_iterator it = allZones.begin(), end = allZones.end(); it != end; ++it )
        {
            zones++;
            printf( "%s\n", it.key().latin1() );
        }
        if (!zones)
        {
            printf( "No timezones in system database\n");
            error++;
        }
        return error ? 1 : 0;
    }

    printf( "Usage: ktimezonestest [local|all]!\n" );
    return error ? 1 : 0;
}
